/*
 Name:		ESP32-3.0.x-TimerTest.ino
 Created:	09.08.2024 11:50:09
 Author:	   Ullis Roboter Seite


 Dieses Programm dient zum Test des neuen ESP32 Timer-API in der Board-Library Version 3.0.x.
 Das Programm ezeugt in regelmigen Abstnden einen Interrupt in dem jeweils ein Pin den Zustand wechselt.

 This program is used to test the new ESP32 Timer-API in the board library version 3.0.x.
 The program generates an interrupt at regular intervals in which a pin changes its state.

 API docs: https://espressif-docs.readthedocs-hosted.com/projects/arduino-esp32/en/latest/api/timer.html
  */

#include <UrsEsp.h>
#include "UrsStringHelper.h"

using namespace UrsCPU;

hw_timer_t* timer1 = nullptr;   // Timer
volatile uint32_t interruptCount = 0; // Interrupt-Counter
uint64_t nextDiplayAt;          // When to generate output (millis)


// interrupt handler
IRAM_ATTR void handleTimerInterrupt()   {
   interruptCount = interruptCount + 1;  // count changes/second
}

void setup() {
   Serial.begin(115200);
   Serial.println("\nESP32-3.0.x-TimerTest");

   // Check reset cause
   auto resetReason = UrsESP.getResetReason(0);
   Serial.printf("CPU0 reset reason: %i (%s) %s\n", resetReason, UrsESP.getResetReasonName(resetReason).c_str(), UrsESP.getResetReasonDescription(resetReason).c_str());

   // If not power-on reset then stop.
   if (resetReason != 1) // Otherwise a new reset will be performed continuously.
      while (1);


   timer1 = timerBegin(1'000'000); // Init the timer. Frequency 1 MHz
   if (timer1)
      Serial.printf("Timer 1 frequency is %lu Hz.\n", timerGetFrequency(timer1));
   else {
      Serial.println("Cannot initialize the timer.\nProgram stops.");
      Serial.flush(); // while(1) blocks output
      while (1);
   }

    Serial.println("Terminate input with (CR)LF");

   Serial.println("'f9999' set frequency and start the timer");
   Serial.println("'start' start the timer");
   Serial.println("'stop' stop the timer");
   Serial.println("'restart' restart the timer");
   Serial.println("'i9999' restart the timer and enable interrupts");
   Serial.println("'o9999' restart the timer and enable one-shot interrupt");
   Serial.println("'n' stops interrupts");

   while (Serial.available()) // clear buffers
      Serial.read();

   SHelper.setThousandSeparator('.');
   nextDiplayAt = millis() + 1000;
}


void loop() {
   if (millis() > nextDiplayAt) {
      Serial.printf("%15s %3lu\n",SHelper.valToStr(timerRead(timer1)).c_str(), interruptCount);
      nextDiplayAt += 1000;
   }


   // handle Serial input
   if (!Serial.available())
      return;

   String inp = Serial.readStringUntil('\n');
   inp.toLowerCase();

   // Frequency setup
   if (inp.startsWith("f")) {
      inp.replace(".", ""); // German thousand separator
      inp.replace(",", ""); // English thousand separator
      inp.replace("'", ""); // C++ thousand separator
      uint32_t frequency = inp.substring(1).toInt(); // Get Value
      timerEnd(timer1);
      timer1 = timerBegin(frequency);
      if (timer1)
         Serial.printf("\nTimer 1 frequency is %s Hz.\n", SHelper.valToStr(timerGetFrequency(timer1)).c_str());
      else {
         Serial.println("\nCannot initialize the timer.\nProgram stops.");
         Serial.flush(); // while(1) blocks output
         while (1);
      }
      return;
   }

   // Timer stop
   if (inp.startsWith("stop")) { // German "stopp"
      timerStop(timer1);
      Serial.println("\nTimer 1 stopped.");
      return;
   }

   // Timer start
   if (inp == "start") {
      timerStart(timer1);
      Serial.println("\nTimer 1 started.");
      return;
   }

   // Timer restart
   if (inp == "restart") {
      timerRestart(timer1);
      auto t = timerRead(timer1);
      Serial.println("\nTimer 1 restarted.");
      Serial.println(SHelper.valToStr(t));
      return;
   }


   // Interrupt setup
   if (inp.startsWith("i")) {
      inp.replace(".", ""); // German thousand separator
      inp.replace(",", ""); // English thousand separator
      inp.replace("'", ""); // C++ thousand separator
      uint32_t alarm = inp.substring(1).toInt(); // Get Value

      timerStop(timer1);
      timerRestart(timer1);
      timerAlarm(timer1, alarm, true, 0);
      timerAttachInterrupt(timer1, handleTimerInterrupt);
      interruptCount = 0;

      Serial.printf("\nInterrupts at %s.\n", SHelper.valToStr(alarm).c_str());

      timerStart(timer1);
      return;
   }

   // One shot setup
   if (inp.startsWith("o")) {
      inp.replace(".", ""); // German thousand separator
      inp.replace(",", ""); // English thousand separator
      inp.replace("'", ""); // C++ thousand separator
      uint32_t alarm = inp.substring(1).toInt(); // Get Value

      timerStop(timer1);
      timerRestart(timer1);
      timerAlarm(timer1, alarm, false, 0);
      timerAttachInterrupt(timer1, handleTimerInterrupt);
      interruptCount = 0;

      Serial.printf("\nOne shot interrupt at %s.\n", SHelper.valToStr(alarm).c_str());

      timerStart(timer1);
      return;
   }

   // Detach Interrupt
   if (inp == "n") {
      timerDetachInterrupt(timer1);
      Serial.println("\nInterrupts stopped.");
      return;
   }
   Serial.println("Invalid command");
}